﻿using AutoMapper;
using IRaCIS.Core.Application.Service.BusinessFilter;
using IRaCIS.Core.Domain.Share;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Localization;
using Panda.DynamicWebApi;
using Panda.DynamicWebApi.Attributes;
using System.Diagnostics.CodeAnalysis;
using ZiggyCreatures.Caching.Fusion;

namespace IRaCIS.Core.Application.Service
{

#pragma warning disable CS8618


    #region 非泛型版本

    [TypeFilter(typeof(UnifiedApiResultFilter))]
    [Authorize, DynamicWebApi]
    public class BaseService : IBaseService, IDynamicWebApi
    {
        public IMapper _mapper { get; set; }

        public IUserInfo _userInfo { get; set; }



        public IStringLocalizer _localizer { get; set; }

        public IWebHostEnvironment _hostEnvironment { get; set; }

        public IFusionCache _fusionCache { get; set; }


        public static IResponseOutput Null404NotFound<TEntity>(TEntity? businessObject) where TEntity : class
        {
            return new ResponseOutput<string>()
            .NotOk($"The query object {typeof(TEntity).Name} does not exist , or was deleted by someone else, or an incorrect parameter query caused", code: ApiResponseCodeEnum.DataNotExist);
        }
    }


    public interface IBaseService
    {
        [MemberNotNull(nameof(_mapper))]
        public IMapper _mapper { get; set; }

        [MemberNotNull(nameof(_userInfo))]
        public IUserInfo _userInfo { get; set; }


        [MemberNotNull(nameof(_localizer))]
        public IStringLocalizer _localizer { get; set; }

        [MemberNotNull(nameof(_hostEnvironment))]
        public IWebHostEnvironment _hostEnvironment { get; set; }

        [MemberNotNull(nameof(_fusionCache))]
        public IFusionCache _fusionCache { get; set; }

    }
    #endregion


    #region 泛型版本测试


    public interface IBaseServiceTest<T> where T : Entity
    {
        [MemberNotNull(nameof(_mapper))]
        public IMapper _mapper { get; set; }

        [MemberNotNull(nameof(_userInfo))]
        public IUserInfo _userInfo { get; set; }

        [MemberNotNull(nameof(_localizer))]
        public IStringLocalizer _localizer { get; set; }

        [MemberNotNull(nameof(_fusionCache))]
        public IFusionCache _fusionCache { get; set; }

    }

    [TypeFilter(typeof(UnifiedApiResultFilter))]
    [Authorize, DynamicWebApi]
    public class BaseServiceTest<T> : IBaseServiceTest<T>, IDynamicWebApi where T : Entity
    {
        public IMapper _mapper { get; set; }

        public IUserInfo _userInfo { get; set; }

        public IStringLocalizer _localizer { get; set; }

        public IFusionCache _fusionCache { get; set; }

        public static IResponseOutput Null404NotFound<TEntity>(TEntity? businessObject) where TEntity : class
        {
            return new ResponseOutput<string>()
            .NotOk($"The query object {typeof(TEntity).Name} does not exist , or was deleted by someone else, or an incorrect parameter query caused", code: ApiResponseCodeEnum.DataNotExist);
        }


    }


    #endregion







}
