﻿using AutoMapper;
using EntityFrameworkCore.Triggered;
using MassTransit;

namespace IRaCIS.Core.Application.Triggers
{
    /// <summary>
    /// 添加访视计划  要给改项目下的所有Subject 添加该访视
    /// </summary>
    public class AddSubjectTrigger(
         IRepository<SubjectVisit> _subjectVisitRepository,

         IRepository<VisitStage> _visitStageRepository,
         IRepository<Trial> _trialRepository,
         IMapper _mapper) : IBeforeSaveTrigger<Subject>
    {


        public async Task BeforeSave(ITriggerContext<Subject> context, CancellationToken cancellationToken)
        {
            var subject = context.Entity;

            if (context.ChangeType == ChangeType.Added)
            {



                //添加受试者的时候，获取访视计划列表，添加到受试者访视表。
                var visitPlanList = await _visitStageRepository.Where(t => t.TrialId == subject.TrialId && t.IsConfirmed).ToListAsync();

                var svList = _mapper.Map<List<SubjectVisit>>(visitPlanList);

                var IsEnrollementQualificationConfirm = await _trialRepository.Where(t => t.Id == subject.TrialId).Select(u => u.IsEnrollementQualificationConfirm).FirstOrDefaultAsync();





                svList.ForEach(t =>
                {
                    t.Subject = subject;
                    t.SubjectId = subject.Id;
                    t.TrialId = subject.TrialId;
                    t.TrialSiteId = subject.TrialSiteId;
                    t.IsEnrollmentConfirm = t.IsBaseLine ? IsEnrollementQualificationConfirm : false;
                    t.Id = NewId.NextGuid();

                });


                await _subjectVisitRepository.AddRangeAsync(svList);

                await _subjectVisitRepository.SaveChangesAsync();



            }
        }
    }
}